@@ -1,15 +1,14 @@
 use bbqueue::framed::FrameConsumer;
 use embedded_hal::serial;
-use fugit::MillisDurationU32;
+use fugit::ExtU32;
 
-use crate::error::Error;
+use crate::error::{Error, Response};
 use crate::helpers::LossyStr;
-use crate::queues::ComProducer;
 use crate::traits::{AtatClient, AtatCmd, AtatUrc};
-use crate::ResponseHeader;
-use crate::{Command, Config};
+use crate::Config;
 
 #[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 enum ClientState {
     Idle,
     AwaitingResponse,
@@ -17,6 +16,7 @@ enum ClientState {
 
 /// Whether the AT client should block while waiting responses or return early.
 #[derive(Debug, Copy, Clone, Hash, PartialEq, Eq)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 pub enum Mode {
     /// The function call will wait as long as necessary to complete the operation
     Blocking,
@@ -48,8 +48,6 @@ pub struct Client<
     res_c: FrameConsumer<'static, RES_CAPACITY>,
     /// The URC consumer receives URCs from the ingress manager
     urc_c: FrameConsumer<'static, URC_CAPACITY>,
-    /// The command producer can send commands to the ingress manager
-    com_p: ComProducer,
 
     state: ClientState,
     timer: CLK,
@@ -66,15 +64,15 @@ where
         tx: Tx,
         res_c: FrameConsumer<'static, RES_CAPACITY>,
         urc_c: FrameConsumer<'static, URC_CAPACITY>,
-        com_p: ComProducer,
-        timer: CLK,
+        mut timer: CLK,
         config: Config,
     ) -> Self {
+        timer.start(config.cmd_cooldown.millis()).ok();
+
         Self {
             tx,
             res_c,
             urc_c,
-            com_p,
             state: ClientState::Idle,
             config,
             timer,
@@ -91,13 +89,8 @@ where
     fn send<A: AtatCmd<LEN>, const LEN: usize>(
         &mut self,
         cmd: &A,
-    ) -> nb::Result<A::Response, Error<A::Error>> {
+    ) -> nb::Result<A::Response, Error> {
         if let ClientState::Idle = self.state {
-            if A::FORCE_RECEIVE_STATE && self.com_p.enqueue(Command::ForceReceiveState).is_err() {
-                // TODO: Consider how to act in this situation.
-                error!("Failed to signal parser to force state transition to 'ReceivingResponse'!",);
-            }
-
             // compare the time of the last response or URC and ensure at least
             // `self.config.cmd_cooldown` ms have passed before sending a new
             // command
@@ -129,9 +122,7 @@ where
             Mode::Blocking => Ok(nb::block!(self.check_response(cmd))?),
             Mode::NonBlocking => self.check_response(cmd),
             Mode::Timeout => {
-                self.timer
-                    .start(MillisDurationU32::from_ticks(A::MAX_TIMEOUT_MS).convert())
-                    .ok();
+                self.timer.start(A::MAX_TIMEOUT_MS.millis()).ok();
                 Ok(nb::block!(self.check_response(cmd))?)
             }
         }
@@ -139,9 +130,7 @@ where
 
     fn peek_urc_with<URC: AtatUrc, F: FnOnce(URC::Response) -> bool>(&mut self, f: F) {
         if let Some(urc_grant) = self.urc_c.read() {
-            self.timer
-                .start(MillisDurationU32::from_ticks(self.config.cmd_cooldown).convert())
-                .ok();
+            self.timer.start(self.config.cmd_cooldown.millis()).ok();
             if let Some(urc) = URC::parse(urc_grant.as_ref()) {
                 if !f(urc) {
                     return;
@@ -156,43 +145,35 @@ where
     fn check_response<A: AtatCmd<LEN>, const LEN: usize>(
         &mut self,
         cmd: &A,
-    ) -> nb::Result<A::Response, Error<A::Error>> {
+    ) -> nb::Result<A::Response, Error> {
         if let Some(mut res_grant) = self.res_c.read() {
             res_grant.auto_release(true);
 
+            let res = match Response::from(res_grant.as_ref()) {
+                Response::Result(r) => r,
+                Response::Prompt(_) => Ok(&[][..]),
+            };
+
             return cmd
-                .parse(ResponseHeader::from_bytes(res_grant.as_ref()))
+                .parse(res)
                 .map_err(nb::Error::from)
                 .and_then(|r| {
                     if let ClientState::AwaitingResponse = self.state {
-                        self.timer
-                            .start(
-                                MillisDurationU32::from_ticks(self.config.cmd_cooldown).convert(),
-                            )
-                            .ok();
+                        self.timer.start(self.config.cmd_cooldown.millis()).ok();
                         self.state = ClientState::Idle;
                         Ok(r)
                     } else {
-                        // FIXME: Is this correct?
-                        error!("Is this correct?! WouldBlock");
                         Err(nb::Error::WouldBlock)
                     }
                 })
                 .map_err(|e| {
-                    self.timer
-                        .start(MillisDurationU32::from_ticks(self.config.cmd_cooldown).convert())
-                        .ok();
+                    self.timer.start(self.config.cmd_cooldown.millis()).ok();
                     self.state = ClientState::Idle;
                     e
                 });
         } else if let Mode::Timeout = self.config.mode {
             if self.timer.wait().is_ok() {
                 self.state = ClientState::Idle;
-                // Tell the parser to reset to initial state due to timeout
-                if self.com_p.enqueue(Command::Reset).is_err() {
-                    // TODO: Consider how to act in this situation.
-                    error!("Failed to signal parser to clear buffer on timeout!");
-                }
                 return Err(nb::Error::Other(Error::Timeout));
             }
         }
@@ -204,10 +185,6 @@ where
     }
 
     fn reset(&mut self) {
-        if self.com_p.enqueue(Command::Reset).is_err() {
-            // TODO: Consider how to act in this situation.
-            error!("Failed to signal ingress manager to reset!");
-        }
         while let Some(grant) = self.res_c.read() {
             grant.release();
         }
@@ -221,15 +198,14 @@ where
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::queues;
     use crate::{self as atat, InternalError};
     use crate::{
         atat_derive::{AtatCmd, AtatEnum, AtatResp, AtatUrc},
-        Clock, GenericError,
+        clock::Clock,
     };
     use bbqueue::framed::FrameProducer;
     use bbqueue::BBBuffer;
-    use heapless::{spsc::Queue, String, Vec};
+    use heapless::String;
     use nb;
 
     const TEST_RX_BUF_LEN: usize = 256;
@@ -266,6 +242,15 @@ mod test {
         }
     }
 
+    #[derive(Debug)]
+    pub enum SerialError {}
+
+    impl embedded_hal::serial::Error for SerialError {
+        fn kind(&self) -> serial::ErrorKind {
+            serial::ErrorKind::Other
+        }
+    }
+
     struct TxMock {
         s: String<64>,
     }
@@ -337,7 +322,7 @@ mod test {
         pub rst: Option<ResetMode>,
     }
     #[derive(Clone, AtatCmd)]
-    #[at_cmd("+CUN", TestResponseStringMixed, timeout_ms = 180000)]
+    #[at_cmd("+CUN", TestResponseStringMixed, timeout_ms = 180000, attempts = 1)]
     pub struct TestRespStringMixCmd {
         #[at_arg(position = 1)]
         pub fun: Functionality,
@@ -415,11 +400,6 @@ mod test {
             static mut URC_Q: BBBuffer<TEST_URC_CAPACITY> = BBBuffer::new();
             let (urc_p, urc_c) = unsafe { URC_Q.try_split_framed().unwrap() };
 
-            static mut COM_Q: queues::ComQueue = Queue::new();
-            let (com_p, _com_c) = unsafe { COM_Q.split() };
-
-            assert_eq!(com_p.capacity(), crate::queues::COM_CAPACITY);
-
             let tx_mock = TxMock::new(String::new());
             let client: Client<
                 TxMock,
@@ -427,25 +407,34 @@ mod test {
                 TIMER_HZ,
                 TEST_RES_CAPACITY,
                 TEST_URC_CAPACITY,
-            > = Client::new(tx_mock, res_c, urc_c, com_p, CdMock, $config);
+            > = Client::new(tx_mock, res_c, urc_c, CdMock, $config);
             (client, res_p, urc_p)
         }};
     }
 
     pub fn enqueue_res(
         producer: &mut FrameProducer<'static, TEST_RES_CAPACITY>,
-        res: Result<Vec<u8, TEST_RX_BUF_LEN>, InternalError>,
+        res: Result<&[u8], InternalError>,
     ) {
-        let (header, bytes) = ResponseHeader::as_bytes(&res);
+        let header: crate::error::Encoded = res.into();
 
-        if let Ok(mut grant) = producer.grant(bytes.len() + header.len()) {
-            grant[0..header.len()].copy_from_slice(&header);
-            grant[header.len()..header.len() + bytes.len()].copy_from_slice(bytes);
-            grant.commit(bytes.len() + header.len());
-        } else {
-            // FIXME: Handle queue being full
-            error!("Response queue full!");
-        }
+        let mut grant = producer.grant(header.len()).unwrap();
+        match header {
+            crate::error::Encoded::Simple(h) => grant[..1].copy_from_slice(&[h]),
+            crate::error::Encoded::Nested(h, b) => {
+                grant[..1].copy_from_slice(&[h]);
+                grant[1..2].copy_from_slice(&[b]);
+            }
+            crate::error::Encoded::Array(h, b) => {
+                grant[..1].copy_from_slice(&[h]);
+                grant[1..header.len()].copy_from_slice(&b);
+            }
+            crate::error::Encoded::Slice(h, b) => {
+                grant[..1].copy_from_slice(&[h]);
+                grant[1..header.len()].copy_from_slice(b);
+            }
+        };
+        grant.commit(header.len());
     }
 
     #[test]
@@ -454,13 +443,10 @@ mod test {
 
         let cmd = ErrorTester { x: 7 };
 
-        enqueue_res(&mut p, Err(InternalError::Error(Vec::new())));
+        enqueue_res(&mut p, Err(InternalError::Error));
 
         assert_eq!(client.state, ClientState::Idle);
-        assert_eq!(
-            nb::block!(client.send(&cmd)),
-            Err(Error::Error(InnerError::Test))
-        );
+        assert_eq!(nb::block!(client.send(&cmd)), Err(Error::Error));
         assert_eq!(client.state, ClientState::Idle);
     }
 
@@ -473,13 +459,10 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        enqueue_res(&mut p, Err(InternalError::Error(Vec::new())));
+        enqueue_res(&mut p, Err(InternalError::Error));
 
         assert_eq!(client.state, ClientState::Idle);
-        assert_eq!(
-            nb::block!(client.send(&cmd)),
-            Err(Error::Error(GenericError))
-        );
+        assert_eq!(nb::block!(client.send(&cmd)), Err(Error::Error));
         assert_eq!(client.state, ClientState::Idle);
     }
 
@@ -492,7 +475,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
+        enqueue_res(&mut p, Ok(&[]));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(client.send(&cmd), Ok(NoResponse));
@@ -504,7 +487,7 @@ mod test {
             "Wrong encoding of string"
         );
 
-        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
+        enqueue_res(&mut p, Ok(&[]));
 
         let cmd = Test2Cmd {
             fun: Functionality::DM,
@@ -519,26 +502,6 @@ mod test {
         );
     }
 
-    #[test]
-    #[ignore]
-    fn countdown() {
-        let (mut client, _, _) = setup!(Config::new(Mode::Timeout));
-
-        assert_eq!(client.state, ClientState::Idle);
-
-        let cmd = Test2Cmd {
-            fun: Functionality::DM,
-            rst: Some(ResetMode::Reset),
-        };
-        assert_eq!(client.send(&cmd), Err(nb::Error::Other(Error::Timeout)));
-
-        match client.config.mode {
-            Mode::Timeout => {} // assert_eq!(cd_mock.time, 180000),
-            _ => panic!("Wrong AT mode"),
-        }
-        assert_eq!(client.state, ClientState::Idle);
-    }
-
     #[test]
     fn blocking() {
         let (mut client, mut p, _) = setup!(Config::new(Mode::Blocking));
@@ -548,7 +511,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
+        enqueue_res(&mut p, Ok(&[]));
 
         assert_eq!(client.state, ClientState::Idle);
         assert_eq!(client.send(&cmd), Ok(NoResponse));
@@ -571,7 +534,7 @@ mod test {
 
         assert_eq!(client.check_response(&cmd), Err(nb::Error::WouldBlock));
 
-        enqueue_res(&mut p, Ok(Vec::<u8, TEST_RX_BUF_LEN>::new()));
+        enqueue_res(&mut p, Ok(&[]));
 
         assert_eq!(client.state, ClientState::AwaitingResponse);
 
@@ -590,8 +553,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        let response =
-            Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+CUN: 22,16,\"0123456789012345\"").unwrap();
+        let response = b"+CUN: 22,16,\"0123456789012345\"";
         enqueue_res(&mut p, Ok(response));
 
         assert_eq!(client.state, ClientState::Idle);
@@ -612,8 +574,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        let response =
-            Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+CUN: \"0123456789012345\",22,16").unwrap();
+        let response = b"+CUN: \"0123456789012345\",22,16";
         enqueue_res(&mut p, Ok(response));
 
         assert_eq!(
@@ -631,7 +592,7 @@ mod test {
     fn urc() {
         let (mut client, _, mut urc_p) = setup!(Config::new(Mode::NonBlocking));
 
-        let response = Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+UMWI: 0, 1").unwrap();
+        let response = b"+UMWI: 0, 1";
 
         let mut grant = urc_p.grant(response.len()).unwrap();
         grant.copy_from_slice(response.as_ref());
@@ -652,7 +613,7 @@ mod test {
             rst: Some(ResetMode::DontReset),
         };
 
-        let response = Vec::<u8, TEST_RX_BUF_LEN>::from_slice(b"+CUN: 22,16,22").unwrap();
+        let response = b"+CUN: 22,16,22";
         enqueue_res(&mut p, Ok(response));
 
         assert_eq!(client.state, ClientState::Idle);
