@@ -1,5 +1,4 @@
 use heapless::{String, Vec};
-use serde_at::CharVec;
 
 /// Trait used by [`atat_derive`] to estimate lengths of the serialized commands, at compile time.
 ///
@@ -58,10 +57,6 @@ where
     const LEN: usize = L * <T as AtatLen>::LEN;
 }
 
-impl<const N: usize> AtatLen for CharVec<N> {
-    const LEN: usize = N;
-}
-
 #[cfg(test)]
 mod tests {
     use std::convert::TryFrom;
