@@ -15,7 +15,7 @@
 //!
 //! ### Command and response example without `atat_derive`:
 //! ```
-//! use atat::{AtatCmd, AtatResp, Error, InternalError, GenericError};
+//! use atat::{AtatCmd, AtatResp, Error, InternalError};
 //! use core::fmt::Write;
 //! use heapless::{String, Vec};
 //!
@@ -37,7 +37,6 @@
 //!
 //! impl<'a> AtatCmd<64> for SetGreetingText<'a> {
 //!     type Response = NoResponse;
-//!     type Error = GenericError;
 //!
 //!     fn as_bytes(&self) -> Vec<u8, 64> {
 //!         let mut buf: Vec<u8, 64> = Vec::new();
@@ -45,20 +44,19 @@
 //!         buf
 //!     }
 //!
-//!     fn parse(&self, resp: Result<&[u8], InternalError>) -> Result<Self::Response, Error<Self::Error>> {
+//!     fn parse(&self, resp: Result<&[u8], InternalError>) -> Result<Self::Response, Error> {
 //!         Ok(NoResponse)
 //!     }
 //! }
 //!
 //! impl AtatCmd<8> for GetGreetingText {
 //!     type Response = GreetingText;
-//!     type Error = GenericError;
 //!
 //!     fn as_bytes(&self) -> Vec<u8, 8> {
 //!         Vec::from_slice(b"AT+CSGT?").unwrap()
 //!     }
 //!
-//!     fn parse(&self, resp: Result<&[u8], InternalError>) -> Result<Self::Response, Error<Self::Error>> {
+//!     fn parse(&self, resp: Result<&[u8], InternalError>) -> Result<Self::Response, Error> {
 //!         // Parse resp into `GreetingText`
 //!         Ok(GreetingText {
 //!             text: String::from(core::str::from_utf8(resp.unwrap()).unwrap()),
@@ -223,15 +221,16 @@ pub(crate) mod fmt;
 
 mod builder;
 mod client;
-mod digest;
+pub mod clock;
+pub mod digest;
 mod error;
 pub mod helpers;
 mod ingress_manager;
 mod queues;
 mod traits;
-mod urc_matcher;
 
 pub use bbqueue;
+pub use nom;
 
 #[cfg(feature = "bytes")]
 pub use serde_bytes;
@@ -255,26 +254,11 @@ pub use heapless;
 
 pub use builder::ClientBuilder;
 pub use client::{Client, Mode};
-pub use digest::{DefaultDigester, DigestResult, Digester};
-pub use error::{Error, GenericError, InternalError};
+pub use digest::{AtDigester, AtDigester as DefaultDigester, DigestResult, Digester, Parser};
+pub use error::{Error, InternalError};
 pub use ingress_manager::IngressManager;
-pub use queues::{ComQueue, Queues};
+pub use queues::Queues;
 pub use traits::{AtatClient, AtatCmd, AtatResp, AtatUrc};
-pub use urc_matcher::{DefaultUrcMatcher, UrcMatcher, UrcMatcherResult};
-
-pub use fugit_timer::Timer as Clock;
-
-/// Commands that can be sent from the client to the ingress manager, for
-/// configuration after initial setup. This is also used for stuff like clearing
-/// the receive buffer on command timeouts.
-#[derive(Debug, Copy, Clone, Hash, PartialEq, Eq)]
-pub enum Command {
-    /// Reset to initial state, including clearing the buffer,
-    /// usually as a result of a command timeout
-    Reset,
-    /// Force the ingress manager into receive state
-    ForceReceiveState,
-}
 
 /// Configuration of both the ingress manager, and the AT client. Some of these
 /// parameters can be changed on the fly, through issuing a [`Command`] from the
@@ -312,28 +296,6 @@ impl Config {
     }
 }
 
-pub struct ResponseHeader;
-
-impl ResponseHeader {
-    pub fn as_bytes<'a, const BUF_LEN: usize>(
-        res: &'a Result<heapless::Vec<u8, BUF_LEN>, InternalError>,
-    ) -> ([u8; 2], &'a [u8]) {
-        match res {
-            Ok(ref r) => ([0xFF, 0xFF], r.as_ref()),
-            Err(InternalError::Error(ref b)) => ([0x00, 0x07], b.as_ref()),
-            Err(ref e) => ([0x00, e.as_byte()], &[]),
-        }
-    }
-
-    pub fn from_bytes(bytes: &[u8]) -> Result<&[u8], InternalError> {
-        match bytes[0] {
-            0xFF => Ok(&bytes[2..]),
-            0x00 => Err(InternalError::from_bytes(&bytes[1..])),
-            _ => Err(InternalError::Parse),
-        }
-    }
-}
-
 #[cfg(test)]
 #[cfg(feature = "defmt")]
 mod tests {
